# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as mpl
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
from matplotlib.ticker import LinearLocator, FormatStrFormatter

font = {'weight' : 'normal','size' : 13}
mpl.rc('font', **font)
fig = {'dpi': 300,'edgecolor': 'w'}
mpl.rc('savefig',**fig)
# mpl.rc('text', usetex=True)

#===============================================================================
def ecriture(fic,x,y,Z):
  """ Ecriture d'un tableau 2D z(x,y) avec entete.
      L'entete comprend deux lignes: xmin xmax xpas / ymin ymax ypas. 
      Les données sont équi-échantillonnées.
  """
  xmin, xmax = np.min(x), np.max(x)
  ymin, ymax = np.min(y), np.max(y)
  xpas, ypas = x[1]-x[0], y[1]-y[0]

  fi = open(fic,'wt')
  fi.write("%f  %f  %f\n" %(xmin,xmax,xpas))
  fi.write("%f  %f  %f\n" %(ymin,ymax,ypas))
  for i in range(Z.shape[0]):
     ligne = ''
     for j in range(Z.shape[1]):
        ligne = ligne + "%f  " %(Z[i,j])
     fi.write("%s\n" %ligne)

  fi.close()

#===============================================================================
def lecture_tsv(fic,lignes_entete=0):
  """ lecture d'un tableau tsv.
      syntaxe: data, entete = lecture_tsv(fic,lignes_entete=0) """
  # fic = "ficZ.tsv"; lignes_entete=2

  if lignes_entete > 0:
     fi = open(fic,'r')
     entete = []
     for i in range(lignes_entete):
        entete.append(fi.readline())
     fi.close()
  else:
     entete = None
     
  data = np.loadtxt(fic,skiprows=lignes_entete)

  return data, entete

#===============================================================================
def contour(nfig,X,Y,Z,filled=False, titre=None,labX=None,labY=None,
            figure_name=None,hold=False):
  """ Tracé de coutours avec éventuellement des vecteurs superposés.
      syntaxe : contourv(nfig,X,Y,Z,filled=None,titre=None,
                         labX="X",labY="Y",figure_name=None,hold=False):
  """
#-------------------------------------------------------------------------------
  mpl.figure(nfig)
  if not hold:
     mpl.clf()

  #mpl.pcolormesh(X,Y,Z)
  if filled == False:
     mpl.contour(X,Y,Z,linewidths=2)
  else:
     mpl.contourf(X,Y,Z)
     mpl.colorbar()

  mpl.axis("equal")
  mpl.axis([X[0,0],X[0,-1],Y[0,0],Y[-1,0]])

  if labX  != None: mpl.xlabel(labX); 
  if labY  != None: mpl.ylabel(labY)
  if titre != None: mpl.title(titre,fontsize=14)

  if not hold:
     mpl.show(block=False)
  else:
     mpl.draw()

  if not figure_name == None:
     mpl.savefig(figure_name)

#===============================================================================
def plot_vecteurs(nfig,X,Y,U,V,ndecim=1,color="black",titre=None,
             labX=None,labY=None,figure_name=None,hold=False):
  """ Tracé de coutours avec éventuellement des vecteurs superposés.
      syntaxe : contourv(nfig,X,Y,U,V,ndecim=1,color='k',titre=None,
                         labX=None,labY=None,figure_name=None,hold=False):
  """
#-------------------------------------------------------------------------------
  mpl.figure(nfig)
  if not hold:
     mpl.clf()

  Q = mpl.quiver(X[::ndecim,::ndecim],Y[::ndecim,::ndecim],
                 U[::ndecim,::ndecim],V[::ndecim,::ndecim],
                 units='dots',angles='xy', 
                 color=color, linewidths=(0.2,),
                 width = 5, headaxislength=2.5
                 )   

  #K = mpl.quiverkey(Q, 0.5, 0.92, 2, r'$2 \frac{m}{s}$', labelpos='W',
                    #fontproperties={'weight': 'bold'})

  mpl.axis('equal')
  mpl.axis([X[0,0],X[0,-1],Y[0,0],Y[-1,0]])
  
  if labX  != None: mpl.xlabel(labX); 
  if labY  != None: mpl.ylabel(labY)
  if titre != None: mpl.title(titre,fontsize=14)

  if not hold:
     mpl.show(block=False)
  else:
     mpl.draw()

  if not figure_name == None:
     mpl.savefig(figure_name)

#===============================================================================
def surf3D(nfig,X,Y,Z,titre="surface 3D",labX=None,labY=None,labZ=None,
           figure_name=None,hold=False):

  fig = mpl.figure(nfig)
  if not hold:
     mpl.clf()

  ax = fig.gca(projection='3d')
  surf = ax.plot_surface(X, Y, Z, rstride=1, cstride=1, cmap=cm.coolwarm,
                         linewidth=0, antialiased=False)
  zmin, zmax = np.min(Z), np.max(Z)
  dz = zmax-zmin
  ax.set_zlim(zmin-dz/100, zmax+dz/100)
  ax.set_xlim(X[0,0],X[0,-1])
  ax.set_ylim(Y[0,0],Y[-1,0])

  ax.zaxis.set_major_locator(LinearLocator(10))
  ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))

  fig.colorbar(surf, shrink=0.5, aspect=5)

  if labX  != None: mpl.xlabel(labX); 
  if labY  != None: mpl.ylabel(labY)
  if titre != None: mpl.title(titre,fontsize=14)

  if not hold:
     mpl.show(block=False)
  else:
     mpl.draw()

  if not figure_name == None:
     mpl.savefig(figure_name)
